/*
	LUSScreenSaver Plugin Interface
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSSSPictureWidget.h"

LUSSSPictureWidget::LUSSSPictureWidget( QWidget* parent, const char* name )
	: QWidget( parent, name, WDestructiveClose )
	, m_nCnt( -1 )
	, m_nInterval( 5000 )
{
	clearWFlags( WStyle_Title );
	m_bLoading = FALSE;
	m_bDo = FALSE;
}

LUSSSPictureWidget::~LUSSSPictureWidget(  )
{
}

void LUSSSPictureWidget::Start()
{
	m_bDo = TRUE;

	setBackgroundColor( QColor( 0, 0, 0 ) );
	erase();
	show();

	QString msg( "...Wait..." );
	QFontMetrics fm( font() );
	QPoint msgPoint( ( width() - fm.width( msg ) ) / 2, ( height() - fm.height() ) / 2 );
	QPainter p;
	p.begin( this );
	p.setPen( QColor( 255, 255, 255 ) );
	p.drawText( msgPoint, msg );
	p.end();

	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "LUSSSPicture" );
	m_nInterval = cfg.readNumEntry( "Interval", 5 ) * 1000;

	if ( m_nCnt == -1 )
	{
		Config cfg( "LUSScreenSaver" );
		cfg.setGroup( "LUSSSPicture" );

		QString path = cfg.readEntry( "Path" );

		if ( path.isEmpty() || ! QFile::exists( path ) )
		{
			path = QPEApplication::documentDir() + "Image_Files/";
		}

		QDir oDir( path );
		oDir.setFilter( QDir::Files | QDir::Hidden );
		oDir.setSorting( QDir::Name );
		oDir.setNameFilter( "*.png;*.PNG;*.gif;*.GIF;*.jpg;*.JPG;*.bmp;*.BMP" );

		const QFileInfoList *pList = oDir.entryInfoList();
		QFileInfoListIterator it( *pList );
		QFileInfo *fi;
		while ( (int)( fi = it.current() ) )
		{
			m_FileList << fi->absFilePath();
			++it;
		}
	}

	if ( m_FileList.count() > 0 )
	{
		QString msg = QString( "%1 images in list" ).arg( m_FileList.count() );
		m_nCnt = 0;
		readyImage();
		QTimer::singleShot( 100, this, SLOT( timerDo() ) );
	}
}

void LUSSSPictureWidget::Stop()
{
	m_bDo = FALSE;
}

void LUSSSPictureWidget::timerDo()
{
	if ( !m_bDo )
	{
		return;
	}

	if ( m_bLoading )
	{
		QTimer::singleShot( 10, this, SLOT( timerDo() ) );
		return;
	}

	erase();
	bitBlt( this, ( width() - m_oBuffer.width() ) / 2, ( height() - m_oBuffer.height() ) / 2, &m_oBuffer );

	QTimer::singleShot( m_nInterval, this, SLOT( timerDo() ) );

	readyImage();
}

void LUSSSPictureWidget::readyImage()
{
	m_bLoading = TRUE;

	QString imgPath = m_FileList[ m_nCnt ];
	
	if ( ++ m_nCnt >= ( signed int )m_FileList.count() )
	{
		m_nCnt = 0;
	}

	m_oBuffer.load( imgPath );
	
	QSize sizeScaling = m_oBuffer.size();

	if ( sizeScaling.height() > height() )
	{
		sizeScaling.setWidth( (double)sizeScaling.width() * ( (double)height() / (double)sizeScaling.height() ) );
		sizeScaling.setHeight( height() );
	}

	if ( sizeScaling.width() > width() )
	{
		sizeScaling.setHeight( (double)sizeScaling.height() * ( (double)width() / (double)sizeScaling.width() ) );
		sizeScaling.setWidth( width() );
	}

	if ( m_oBuffer.size() != sizeScaling )
	{
		m_oBuffer = m_oBuffer.smoothScale( sizeScaling.width(), sizeScaling.height() );
	}
	
	m_bLoading = FALSE;
}

